/*
 * @copyright		 Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.multistore.ui.affiliateCatalogBrowser
 *
 * @revision		$Revision: 1.10 $
 */

define("de_epages/multistore/ui/affiliatecatalogbrowser", [
	"jquery",
	"ep",
	"de_epages",
	"$dict!../dictionary",
	"$tmpl!./affiliatecatalogbrowser",
	"$tmpl!./affiliatecatalogbrowser-categorynode",
	"$tmpl!./affiliatecatalogbrowser-productnode",

	"jquery/jstree",
	"jquery/jstree/themes",
	"jquery/jstree/json_data",
	"jquery/jstree/ui",
	"jquery/jstree/crrm",
	"jquery/jstree/dnd",
	"jquery/effects/fade",
	"ep/ajax",
	"ep/alert",
	"ep/fn/busy",
	"ep/fn/contextorientation",
	"ep/ui/input",
	"ep/ui/tooltip"
], function ($, ep, de_epages, dict, tmplCatalogbrowser, tmplCatalogbrowserCategory, tmplCatalogbrowserProduct) {

	/*
	 * @dictionary				ep.dict
	 *
	 * @translation				{Delete}
	 *							{Edit}
	 *
	 *
	 * @dictionary				de_epages.multistore.dictionary
	 *
	 * @translation				{OBJECT_EXISTS}
	 *							{FeatureLimitExceeded}
	 *							{SAVE_ERROR}
	 *							{DELETE_ERROR}
	 *							{NEWOBJECT_ERROR}
	 *							{CATEGORYLIST_ERROR}
	 *							{PRODUCTLIST_ERROR}
	 *							{AddSubCategory}
	 */

	$.widget( 'ui.multistoreUiAffiliateCatalogBrowser',/* my.parent.widget,*/ {
		_categoryList           : null,
		_productList            : null,
		_categoryBreadCrumb     : null,
		_pagerContainer         : null,
		_pagerBack              : null,
		_pagerNext              : null,
		_tree                   : null,
		_selectedTreeNode       : null,
		_selectedTreeNodeParent : null,
		_editButtons            : null,
		_editButton             : null,
		_deleteButton           : null,
		_newButton              : null,
		_textInput              : null,
		_editOpen               : false,
		_oldInputValue          : null,
		_currentPage            : 1,
		_currentId              : null,
		_imageMap: {
			'category': epConfig.javascriptRoot + '/de_epages/multistore/images/ico_m_folder.png'
		},
		options: {
			startpageId : null
		},

		_create: function(){
			var self = this;
			tmplCatalogbrowser({})
				.appendTo(self.element);

			self.dict = dict;

			self.elem = self.element
				.sub(ep)
				.on('mouseenter',$.proxy(self,'_onMouseEnter'));

			self._categoryList = self.elem
				.find('.de_epages-acb-categorylist')
				.busy('show');
			self._productList = self.elem
				.find('.de_epages-acb-productlist')
				.busy('show');
			self._categoryBreadCrumb = self.elem
				.find('.de_epages-acb-breadcrumb')
				.busy('show');
			self._leftContainer = self.elem
				.find('.de_epages-acb-leftcontainer');
			self._pagerBack = self.elem
				.find('.de_epages-acb-pager-back')
				.on('click', function(){
					if(!$(this).hasClass('Disabled')){
						self._buildProductList(self._currentId,self._currentPage - 1);
					}
				})
				.addClass('Disabled');
			self._pagerNext = self.elem
				.find('.de_epages-acb-pager-next')
				.on('click', function(){
					if(!$(this).hasClass('Disabled')){
						self._buildProductList(self._currentId,self._currentPage + 1);
					}
				})
				.addClass('Disabled');

			self._tree = self.elem
				.find('.de_epages-acb-tree')
				.on("create.jstree", $.proxy(self,'_onTreeCreate'));
			self._editButtons = self.elem
				.children('.de_epages-acb-editbuttons')
				.appendTo('body')
				.on('mouseenter',$.proxy(self,'_onMouseEnterEditButtons'));

			self._editButton = self._editButtons
				.children('.de_epages-acb-editbuttons-edit')
				.on('click',$.proxy(self,'_onClickEditButton'));
			self._editButton
				.attr('title',self.dict.translate(self._editButton.attr('title')));
			self._deleteButton = self._editButtons
				.children('.de_epages-acb-editbuttons-delete')
				.on('click',$.proxy(self,'_onClickDeleteButton'));
			self._deleteButton
				.attr('title',self.dict.translate(self._deleteButton.attr('title')));
			self._newButton = self._editButtons
				.children('.de_epages-acb-editbuttons-new')
				.on('click',$.proxy(self,'_onClickNewButton'));
			self._newButton
				.attr('title',self.dict.translate(self._newButton.attr('title')));
			self._textInput = self._editButtons
				.children('.de_epages-acb-textinput')
				.on('blur',$.proxy(self,'_closeEditMode'))
				.on('keypress',$.proxy(self,'_OnKeyPressTextInput'))
				.uiInput({});

			self.elem
				.find('.de_epages-acb-breadcrumb-main')
				.on('click',$.proxy(self,'_onClickBreadCrumb'));

			//build master shop category & product list for the main category
			self._buildCategoryList();
			self._buildProductList();

				//buils shop tree
				self._tree.jstree({
					//config for tree themes plugin
					themes : {
						theme : 'classic',
						dots  : true,
						icons : true
					},
					//config for tree json_data plugin
					json_data : {
						ajax : {
							url : '?',
							type: 'POST',
							dataType: 'json',
							beforeSend: function( xhr, s ){
								xhr.setRequestHeader('Accept', 'text/x-json');
							},
							context: this,
							data: $.proxy(self,'_generateTreeJSONData')
						}
					},
					//config for tree drag and drop plugin
					dnd:{
						open_timeout: 86400000, // we dont want to open the node on hover but we can not disable this function
																				// so we had to choose a high value - 1 day in milliseconds
						drop_target: '.Category',
						drag_finish: $.proxy(self,'_onTreeDragFinish'),
						drag_check: $.proxy(self,'_onTreeDragCheck')
					},
					//config for tree creating renaming removing moving (crrm) plugin
					crrm:{
						 move:{
						 	check_move: function(){ return false;}
						}
					},
					ui:{
						select_limit: 0,
						select_prev_on_delete: false
					},
					plugins : ['themes', 'json_data', 'crrm', 'dnd','ui']
				});

			self._tree
				.on('click', 'a', $.proxy(self,'_onClickTreeNode'))
				.on('mouseenter', 'a', $.proxy(self,'_onMouseEnterTreeNode'))
				.on('mouseleave', 'a', $.proxy(self,'_onMouseLeaveTreeNode'));

			self.elem.dictParse(self.dict, true);
		},

		_generateTreeJSONData: function(node){
			var returnData = {
				ViewAction : 'JSONChildrenJSTree'
			};
			if(node == -1){
				returnData.WithParent = 1;
				returnData.ObjectID = this.options.startpageId;
			}
			else{
				returnData.ObjectID = node.data('jstree').id;
			}
			return returnData;
		},

		_onMouseEnter: function(evt){
			if(!this._editOpen){
				this._editButtons.delay(500).fadeOut(0);
			}
		},

		//Tree Events
		_onTreeCreate: function(evt){
			this._openEditMode(this._selectedTreeNode);
			this._textInput.one('closeEditMode',$.proxy(this,'_createCategory'));
		},

		_onMouseEnterEditButtons: function(evt){
			this._editButtons.stop(true,true).show();
		},

		_onClickEditButton: function(evt){
			var self = this;
			self._openEditMode(self._selectedTreeNode);
			self._textInput.one('closeEditMode',function(evt,undoChanges){
				if(!undoChanges){
					self._tree.busy('show');
					var nodeData = $(self._selectedTreeNode).parent('li').data('jstree'),
					newName = self._tree.jstree('get_text',self._selectedTreeNode);

					ep.ajax({
						dataType: 'json',
						data: {
							ViewAction   : 'JSONViewResponse',
							ObjectID     : nodeData.id,
							Name         : newName,
							ChangeAction : 'Save'
						},
						success: function(data){
							self._tree.busy('hide');
						},
						error: function(xhr,status,error){
							self._showError('SAVE_ERROR');
						}
					});
				}
			});
		},

		_openEditMode:function(node){
			var self = this;
			if(!self._editOpen){
				self._selectedTreeNode = node;
				self._oldInputValue = self._tree.jstree('get_text',node);
				self._tree.jstree('set_text',node,'');
				self._textInput
					.show()
					.val(self._oldInputValue);
				self._editButtons.contextOrientation(node,'right');
				self._editButton.hide();
				self._newButton.hide();
				self._deleteButton.hide();
				self._editOpen = true;
				self._textInput
					.trigger('focus')
					.select();
			}
		},

		_closeEditMode: function( undoChanges){
			var self = this;
			if(self._editOpen){
				self._textInput.hide();
				var newName = undoChanges === true ? self._oldInputValue : self._textInput.val();
				self._tree.jstree('set_text',self._selectedTreeNode,newName);
				self._editOpen = false;
				self._editButtons.contextOrientation(self._selectedTreeNode,'right');
				self._textInput.trigger('closeEditMode',[undoChanges]);
			}
		},

		_refreshNode: function( node, forceOpen ){
			var self = this,
				currentState = self._tree.jstree('is_open',node),
				currentNode = node;
			self._editButtons.hide();
			self._tree
				.busy('show');
			if(self._tree.jstree('is_open ',node)){
				self._tree
				.jstree('close_node',node,false,true);
			}
			self._tree
				.jstree('refresh',node)
				.one('refresh.jstree',function(){
					self._tree.busy('hide');
					if(currentState || forceOpen){
						self._tree.jstree('open_node',currentNode,false,true);
					}
			});
		},

		_onClickDeleteButton: function(evt){
			var self = this,
				nodeData = $(this._selectedTreeNode)
					.parent('li')
					.data('jstree');
			self._editButtons.hide();
			self._tree.busy('show');

				ep.ajax({
					dataType: 'json',
					data: {
						ViewAction   : 'JSONViewResponse',
						ObjectID     : nodeData.id,
						ParentID     : nodeData.parentId,
						ACCEPT       : 1,
						ChangeAction : 'Delete'
					},
					success: function(data){
						self._tree
							.jstree('remove',self._selectedTreeNode)
							.busy('hide');
					},
					error: function(xhr,status,error){
						self._showError('DELETE_ERROR');
					}
				});
		},

		_onClickNewButton: function(evt){
			this._tree
				.jstree('create',this._selectedTreeNode,'inside',{
					data: 'new category'
				},$.proxy(this,'_afterTreeCreateCategory'),true);
		},

		_afterTreeCreateCategory:function(node){
			this._selectedTreeNodeParent = $(this._selectedTreeNode).parent('li');
			this._selectedTreeNode = node.find('a');
		},

		_createCategory: function(){
			var self = this,
				newName = self._tree.jstree('get_text',self._selectedTreeNode);
			if(newName == 'new category'){
				self._tree.jstree('remove',self._selectedTreeNode);
			}
			else{
				var parentNodeData = self._selectedTreeNodeParent.data('jstree');
				ep.ajax({
					dataType: 'json',
					data: {
						ViewAction   : 'JSONViewResponse',
						ObjectID     : parentNodeData.id,
						ChangeAction : 'NewObject',
						ClassID      : self.options.categoryClassId,
						Name         : newName
					},
					success: function(data){
						self._refreshNode(self._selectedTreeNodeParent);
					},
					error: function(xhr,status,error){
						self._showError('NEWOBJECT_ERROR');
					}
				});
			}
		},

		_OnKeyPressTextInput: function(evt){
			if(evt.keyCode == $.ui.keyCode.ENTER){
				this._closeEditMode();
			}
			else if(evt.keyCode == $.ui.keyCode.ESCAPE){
				this._closeEditMode(true);
			}
		},

		_onClickTreeNode: function(evt){
			this._closeEditMode();
			evt.stopPropagation();
			this._tree.jstree('toggle_node',evt.currentTarget);
		},

		_onMouseEnterTreeNode: function(evt){
			var self = this;
			if(!self._editOpen){
				var node = evt.currentTarget,
					nodeType = $(node).parent('li').data('jstree').type;
				self._selectedTreeNode = node;
				self._editButtons
					.stop(true,true)
					.show()
					.contextOrientation(node,'right');

				if(nodeType == 'Category'){
					self._editButton.show();
					self._newButton.show();
					self._deleteButton.show();
				}
				else if(nodeType == undefined){//on StartPage the nodeType is undefined
					self._newButton.show();
					self._editButton.hide();
					self._deleteButton.hide();
				}
				else if(nodeType == 'SubShopCategory' || nodeType == 'SubShopProduct'){
					self._deleteButton.show();
					self._editButton.hide();
					self._newButton.hide();
				}
				else{
					self._editButton.hide();
					self._newButton.hide();
					self._deleteButton.hide();
				}
			}
		},

		_onMouseLeaveTreeNode: function(evt){
			if(!this._editOpen){
				this._editButtons.delay(500).fadeOut(0);
			}
		},

		//DND Events for Tree
		_onTreeDragFinish: function(data){
			var self = this,
				dragSource = $(data.o).is('li') ? $(data.o) : $(data.o).parents('li'),
				dragTarget = data.r,
				dragSourceData = dragSource.tmplItem();
			if(dragSourceData.data){
				self._tree
					.busy('show')
					.one('cloneToSubShop/finish',function(){
						self._refreshNode(dragTarget,true);
					});
				self._cloneToSubShop( dragSourceData.data.path, dragSourceData.data.id, dragTarget.data('jstree').id);
			}
		},

		_onTreeDragCheck: function(data){
			var dragTarget = data.r,
				nodeType = dragTarget ? dragTarget.data('jstree') : false; //if there is node data it is the startpage
				return {
					after: false,
					before: false,
					inside: nodeType && (nodeType.type == 'Category' || nodeType.type == undefined) ? true : false
				};
		},

		_onClickCategoryListItem: function(evt){
			evt.stopPropagation();
			var self = this,
				node = evt.currentTarget,
				objectId = $(node).tmplItem().data.id;
			self._currentId = objectId;
			self._buildCategoryList(objectId);
			self._buildProductList(objectId);
			self._addBreadCrumb(node);
		},

		_onClickBreadCrumb: function(evt){
			evt.stopPropagation();
			var self = this,
				node = $(evt.currentTarget),
					objectId = node.data('acb-objectId');
				node.nextAll()
					.remove();
			if(objectId){
				self._currentId = objectId;
				self._buildCategoryList(self._currentId);
				self._buildProductList(self._currentId);
			}
			else{
				self._buildCategoryList();
				self._buildProductList();
			}
		},

		_addBreadCrumb: function(node){
			var nodeData = $(node)
				.data()
				.tmplItem
				.data;
			$('<a>')
				.attr('href','javascript:void(0);')
				.data('acb-objectId',nodeData.id)
				.html('&nbsp;&gt;&nbsp;' + nodeData.name)
				.appendTo(this._categoryBreadCrumb)
				.on('click',$.proxy(this,'_onClickBreadCrumb'));
		},

		_cloneToSubShop: function(sourceObjectPath, sourceObjectId, targetObjectId){
			var self = this;
			ep.ajax({
				type: 'POST',
				dataType: 'json',
				data: {
					ViewAction        : 'JSONViewResponse',
					ChangeAction      : 'CloneAffiliateObjectToSubShop',
					ObjectID          : ep.config.siteId,
					MasterObjectPath  : sourceObjectPath,
					AffiliateObjectID : sourceObjectId,
					SubShopParentID   : targetObjectId
				},
				success: function(data){
					self._tree.trigger('cloneToSubShop/finish');
				},
				error: function(xhr,status,error){
					var errorData = error[0];
					if (errorData.Reason){
						self._tree.busy('hide');
						self._showError(errorData.Reason);
					} else {
						self._tree.busy('hide');
						self._showError(error + ': Object not exported?');
					}
				}
			});
		},

		_buildCategoryList:function(objectId){
			var self = this;
			self._categoryList.busy('show');
			self._categoryBreadCrumb.busy('show');

			ep.ajax({
					url : this.options.masterWebUrl,
					type: 'GET',
					dataType: 'jsonp',
					data: {
						'ViewAction' : 'JSONMasterSubCategories',
						'MasterCategoryId': objectId,
						'SubShopAlias' : this.options.subShopAlias,
						'LanguageId' : this.options.languageId,
					},
					success: function(data){
						self._categoryList
							.empty()
							.html('<span class="NoSubcategories">' + self.dict.translate('NoSubcategories') + '</span>');
						if(data.children.length){
							self._categoryList.empty();
							for(var i = 0,iLength = data.children.length; i < iLength;i++){
								var childData = data.children[i];
								childData.image = self._imageMap['category'];
								tmplCatalogbrowserCategory(childData )
									.appendTo(self._categoryList);
							}

							self._categoryList
								.children('.de_epages-acb-categorynode')
								.addClass('jstree-draggable');
								self._categoryList
									.children('.de_epages-acb-categorynode')
									.on('click',$.proxy(self,'_onClickCategoryListItem'));
							}
							self._categoryList.busy('hide');
							self._categoryBreadCrumb.busy('hide');
					},
					error: function(xhr,status,error){
						self._categoryList.busy('hide');
						self._categoryBreadCrumb.busy('hide');
						self._showError('CATEGORYLIST_ERROR');
					}
				});
		},

		_buildProductList: function(objectId,page){
			var self = this,
		  pageNumber = page ? page : 1;
			self._productList.busy('show');

			ep.ajax({
					url : this.options.masterWebUrl,
					type: 'GET',
					dataType: 'jsonp',
					data: {
						'ViewAction' : 'JSONMasterPagedProducts',
						'Page' : pageNumber,
						'MasterCategoryId': objectId,
						'SubShopAlias' : this.options.subShopAlias,
						'LanguageId' : this.options.languageId,
					},
					success: function(data){
						var products = data.products;
						self._productList
							.empty()
							.html('<span class="NoProducts">' + self.dict.translate('NoProducts') + '</span>');
						if(products.length){
							self._productList.empty();
							for(var i = 0,iLength = products.length; i < iLength; i++){
								var productData = products[i];
								tmplCatalogbrowserProduct(productData )
									.appendTo(self._productList);
							}
							self._productList
								.busy('hide')
								.children('.de_epages-acb-productnode')
									.addClass('jstree-draggable')
									.each(function(index,elem){
										var node = ep(this),
											desciptionNode = node.find('.de_epages-acb-productnode-description'),
											bubbleNode = node.find('.de_epages-acb-productnode-bubble');
										desciptionNode.uiTooltip({
											context: bubbleNode,
											orientation: 'right'
										});
									});
						}
						var pages = data.pages,
							nextPage = data.nextPage,
							previousPage = data.previousPage;
						self._currentPage = parseInt(data.currentPage);
						if(pages && pages > 1){
							if(nextPage && self._currentPage && (self._currentPage != nextPage)){
								self._pagerNext.removeClass('Disabled');
							}
							else{
								self._pagerNext.addClass('Disabled');
							}
							if(previousPage && self._currentPage && (self._currentPage != previousPage)){
								self._pagerBack.removeClass('Disabled');
							}
							else{
								self._pagerBack.addClass('Disabled');
							}
						}
					},
					error: function(xhr,status,error){
						self._showError('PRODUCTLIST_ERROR');
					}
				});
		},
		_showError: function(errorCode){
			ep.alert({
				type:		'error',
				content:	this.dict.translate(errorCode),
				ok: $.noop
			});
		}
	});

	return de_epages;

});